<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width,initial-scale=1,shrink-to-fit=no">
    <meta name="theme-color" content="#000000">
    <meta http-equiv="Cache-Control" content="no-cache, no-store, must-revalidate" />
    <meta http-equiv="Pragma" content="no-cache" />
    <meta http-equiv="Expires" content="0" />
    <link rel="shortcut icon" type="image/png" href="/static/favicon.724bc5764d3a.ico" />
    <title>Workbench</title>
    <link href="https://fonts.googleapis.com/css?family=Roboto:400,400i,500,500i,700,900&display=swap" rel="stylesheet">
    <script src="https://cdn.plot.ly/plotly-latest.min.js"></script>
    <link href="https://atonra-stainly-cdn.fra1.cdn.digitaloceanspaces.com/test/css/main-0-1-2.css" rel="stylesheet"
        type="text/css">
    <script type="text/javascript">window.bridger_config_url = "http://localhost:5000/bridger/config/"
    </script>
</head>

<body><noscript>You need to enable JavaScript to run this app.</noscript>
    <div id="root"></div>
    <script type="application/javascript"
        src="https://atonra-stainly-cdn.fra1.cdn.digitaloceanspaces.com/test/js/main-0-2-2.js"></script>
</body>

</html>